@@ -2,6 +2,7 @@ import asyncio
 import asyncio.events
 import functools
 import inspect
+import io
 import os
 import re
 import sys
@@ -11,10 +12,37 @@ from glob import has_magic
 
 from .callbacks import _DEFAULT_CALLBACK
 from .exceptions import FSTimeoutError
-from .spec import AbstractFileSystem
+from .spec import AbstractBufferedFile, AbstractFileSystem
 from .utils import is_exception, other_paths
 
 private = re.compile("_[^_]")
+iothread = [None]  # dedicated fsspec IO thread
+loop = [None]  # global event loop for any non-async instance
+_lock = None  # global lock placeholder
+
+
+def get_lock():
+    """Allocate or return a threading lock.
+
+    The lock is allocatted on first use to allow setting one lock per forked process.
+    """
+    global _lock
+    if not _lock:
+        _lock = threading.Lock()
+    return _lock
+
+
+def reset_lock():
+    """Reset the global lock.
+
+    This should be called only on the init of a forked process to reset the lock to
+    None, enabling the new forked process to get a new lock.
+    """
+    global _lock
+
+    iothread[0] = None
+    loop[0] = None
+    _lock = None
 
 
 async def _runner(event, coro, result, timeout=None):
@@ -32,6 +60,9 @@ async def _runner(event, coro, result, timeout=None):
 def sync(loop, func, *args, timeout=None, **kwargs):
     """
     Make loop run coroutine until it returns. Runs in other thread
+
+    Example usage:
+        fsspec.asyn.sync(fsspec.asyn.get_loop(), func, *args, timeout=timeout, **kwargs)
     """
     timeout = timeout if timeout else None  # convert 0 or 0.0 to None
     # NB: if the loop is not running *yet*, it is OK to submit work
@@ -67,11 +98,6 @@ def sync(loop, func, *args, timeout=None, **kwargs):
         return return_result
 
 
-iothread = [None]  # dedicated fsspec IO thread
-loop = [None]  # global event loop for any non-async instance
-lock = threading.Lock()  # for setting exactly one thread
-
-
 def sync_wrapper(func, obj=None):
     """Given a function, make so can be called in async or blocking contexts
 
@@ -120,7 +146,7 @@ def get_loop():
     The loop will be running on a separate thread.
     """
     if loop[0] is None:
-        with lock:
+        with get_lock():
             # repeat the check just in case the loop got filled between the
             # previous two calls from another thread
             if loop[0] is None:
@@ -137,7 +163,7 @@ def get_loop():
 def fsspec_loop():
     """Temporarily switch the current event loop to the fsspec's
     own loop, and then revert it back after the context gets
-    terinated.
+    terminated.
     """
     try:
         original_loop = get_running_loop()
@@ -307,7 +333,7 @@ class AsyncFileSystem(AbstractFileSystem):
         batch_size = batch_size or self.batch_size
         path = await self._expand_path(path, recursive=recursive)
         return await _run_coros_in_chunks(
-            [self._rm_file(p, **kwargs) for p in path],
+            [self._rm_file(p, **kwargs) for p in reversed(path)],
             batch_size=batch_size,
             nofiles=True,
         )
@@ -740,6 +766,11 @@ class AsyncFileSystem(AbstractFileSystem):
     async def _makedirs(self, path, exist_ok=False):
         pass  # not necessary to implement, may not have directories
 
+    async def open_async(self, path, mode="rb", **kwargs):
+        if "b" not in mode or kwargs.get("compression"):
+            raise ValueError
+        raise NotImplementedError
+
 
 def mirror_sync_methods(obj):
     """Populate sync and async methods for obj
@@ -808,3 +839,120 @@ def _dump_running_tasks(
             except exc:
                 pass
     return out
+
+
+class AbstractAsyncStreamedFile(AbstractBufferedFile):
+    # no read buffering, and always auto-commit
+    # TODO: readahead might still be useful here, but needs async version
+
+    async def read(self, length=-1):
+        """
+        Return data from cache, or fetch pieces as necessary
+
+        Parameters
+        ----------
+        length: int (-1)
+            Number of bytes to read; if <0, all remaining bytes.
+        """
+        length = -1 if length is None else int(length)
+        if self.mode != "rb":
+            raise ValueError("File not in read mode")
+        if length < 0:
+            length = self.size - self.loc
+        if self.closed:
+            raise ValueError("I/O operation on closed file.")
+        if length == 0:
+            # don't even bother calling fetch
+            return b""
+        out = await self._fetch_range(self.loc, self.loc + length)
+        self.loc += len(out)
+        return out
+
+    async def write(self, data):
+        """
+        Write data to buffer.
+
+        Buffer only sent on flush() or if buffer is greater than
+        or equal to blocksize.
+
+        Parameters
+        ----------
+        data: bytes
+            Set of bytes to be written.
+        """
+        if self.mode not in {"wb", "ab"}:
+            raise ValueError("File not in write mode")
+        if self.closed:
+            raise ValueError("I/O operation on closed file.")
+        if self.forced:
+            raise ValueError("This file has been force-flushed, can only close")
+        out = self.buffer.write(data)
+        self.loc += out
+        if self.buffer.tell() >= self.blocksize:
+            await self.flush()
+        return out
+
+    async def close(self):
+        """Close file
+
+        Finalizes writes, discards cache
+        """
+        if getattr(self, "_unclosable", False):
+            return
+        if self.closed:
+            return
+        if self.mode == "rb":
+            self.cache = None
+        else:
+            if not self.forced:
+                await self.flush(force=True)
+
+            if self.fs is not None:
+                self.fs.invalidate_cache(self.path)
+                self.fs.invalidate_cache(self.fs._parent(self.path))
+
+        self.closed = True
+
+    async def flush(self, force=False):
+        if self.closed:
+            raise ValueError("Flush on closed file")
+        if force and self.forced:
+            raise ValueError("Force flush cannot be called more than once")
+        if force:
+            self.forced = True
+
+        if self.mode not in {"wb", "ab"}:
+            # no-op to flush on read-mode
+            return
+
+        if not force and self.buffer.tell() < self.blocksize:
+            # Defer write on small block
+            return
+
+        if self.offset is None:
+            # Initialize a multipart upload
+            self.offset = 0
+            try:
+                await self._initiate_upload()
+            except:  # noqa: E722
+                self.closed = True
+                raise
+
+        if self._upload_chunk(final=force) is not False:
+            self.offset += self.buffer.seek(0, 2)
+            self.buffer = io.BytesIO()
+
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        await self.close()
+
+    async def _fetch_range(self, start, end):
+        raise NotImplementedError
+
+    async def _initiate_upload(self):
+        raise NotImplementedError
+
+    async def _upload_chunk(self, final=False):
+        raise NotImplementedError
