@@ -1,12 +1,15 @@
 import os
-import shutil
 import pickle
-import pytest
+import shutil
 import tempfile
 
+import pytest
+
 import fsspec
-from fsspec.implementations.cached import CachingFileSystem
 from fsspec.compression import compr
+from fsspec.exceptions import BlocksizeMismatchError
+from fsspec.implementations.cached import CachingFileSystem, LocalTempFile
+
 from .test_ftp import FTPFileSystem
 
 
@@ -178,6 +181,7 @@ def test_pop():
     with pytest.raises(PermissionError):
         fs.pop_from_cache(f1)
     fs.pop_from_cache(f2)
+    fs.pop_from_cache(os.path.join(origin, "uncached-file"))
     assert len(os.listdir(cache2)) == 1
     assert not fs._check_file(f2)
     assert fs._check_file(f1)
@@ -187,13 +191,15 @@ def test_write_pickle_context():
     tmp = str(tempfile.mkdtemp())
     fn = tmp + "afile"
     url = "simplecache::file://" + fn
-    f = fsspec.open(url, "wb").open()
-    f.write(b"hello ")
-    f.flush()
-    with pickle.loads(pickle.dumps(f)) as f2:
-        f2.write(b"world")
+    with fsspec.open(url, "wb") as f:
+        pickle.loads(pickle.dumps(f))
+        f.write(b"hello ")
+        pickle.dumps(f)
 
-    assert open(fn, "rb").read() == b"hello world"
+    with pytest.raises(ValueError):
+        pickle.dumps(f)
+
+    assert open(fn, "rb").read() == b"hello "
 
 
 def test_blocksize(ftp_writable):
@@ -210,7 +216,7 @@ def test_blocksize(ftp_writable):
 
     with fs.open("/out_block", block_size=20) as f:
         assert f.read(1) == b"t"
-    with pytest.raises(ValueError):
+    with pytest.raises(BlocksizeMismatchError):
         fs.open("/out_block", block_size=30)
 
 
@@ -310,6 +316,35 @@ def test_local_filecache_creates_dir_if_needed(impl):
     assert data_in_cache == data
 
 
+@pytest.mark.parametrize("toplevel", [True, False])
+@pytest.mark.parametrize("impl", ["filecache", "simplecache", "blockcache"])
+def test_get_mapper(impl, toplevel):
+    import tempfile
+
+    original_location = tempfile.mkdtemp()
+    cache_location = tempfile.mkdtemp()
+    os.rmdir(cache_location)
+    original_file = os.path.join(original_location, "afile")
+    data = b"test data"
+    with open(original_file, "wb") as f:
+        f.write(data)
+
+    if toplevel:
+        m = fsspec.get_mapper(
+            f"{impl}::file://{original_location}",
+            **{impl: {"cache_storage": cache_location}},
+        )
+    else:
+        fs = fsspec.filesystem(
+            impl, target_protocol="file", cache_storage=cache_location
+        )
+        m = fs.get_mapper(original_location)
+
+    assert m["afile"] == data
+    assert os.listdir(cache_location)
+    assert m["afile"] == data
+
+
 def test_local_filecache_basic(local_filecache):
     data, original_file, cache_location, fs = local_filecache
 
@@ -414,7 +449,7 @@ def test_filecache_multicache():
     assert fs.cat(f1) == data
 
     assert len(os.listdir(cache1)) == 2  # cache and hashed afile
-    assert len(os.listdir(cache2)) == 0  # hasn't been intialized yet
+    assert len(os.listdir(cache2)) == 0  # hasn't been initialized yet
 
     # populates last cache if file not found in first cache
     fs = fsspec.filesystem(
@@ -497,6 +532,8 @@ def test_filecache_with_checks():
 
     assert fs.cat(f1) == data  # does not change
     assert fs2.cat(f1) == data * 2  # changed, since origin changed
+    with fs2.open(f1) as f:
+        assert f.read() == data * 2  # read also sees new data
     time.sleep(0.11)  # allow cache details to expire
     assert fs.cat(f1) == data * 2  # changed, since origin changed
 
@@ -534,6 +571,25 @@ def test_add_file_to_cache_after_save(local_filecache):
     assert len(fs2.cached_files[-1]) == 1
 
 
+def test_cached_open_close_read(ftp_writable):
+    # Regression test for <https://github.com/fsspec/filesystem_spec/issues/799>
+    host, port, user, pw = ftp_writable
+    fs = FTPFileSystem(host, port, user, pw)
+    with fs.open("/out_block", "wb") as f:
+        f.write(b"test" * 4000)
+    fs = fsspec.filesystem(
+        "cached",
+        target_protocol="ftp",
+        target_options={"host": host, "port": port, "username": user, "password": pw},
+    )
+    with fs.open("/out_block", block_size=1024) as f:
+        pass
+    with fs.open("/out_block", block_size=1024) as f:
+        assert f.read(1) == b"t"
+    # Regression test for <https://github.com/fsspec/filesystem_spec/issues/845>
+    assert fs.cached_files[-1]["/out_block"]["blocks"] == {0}
+
+
 @pytest.mark.parametrize("impl", ["filecache", "simplecache"])
 @pytest.mark.parametrize("compression", ["gzip", "bz2"])
 def test_with_compression(impl, compression):
@@ -549,7 +605,7 @@ def test_with_compression(impl, compression):
         "%s::%s" % (impl, fn),
         "rb",
         compression=compression,
-        **{impl: dict(same_names=True, cache_storage=cachedir)}
+        **{impl: dict(same_names=True, cache_storage=cachedir)},
     ) as f:
         # stores original compressed file, uncompress on read
         assert f.read() == data
@@ -561,7 +617,9 @@ def test_with_compression(impl, compression):
     with fsspec.open(
         "%s::%s" % (impl, fn),
         "rb",
-        **{impl: dict(same_names=True, compression=compression, cache_storage=cachedir)}
+        **{
+            impl: dict(same_names=True, compression=compression, cache_storage=cachedir)
+        },
     ) as f:
         # stores uncompressed data
         assert f.read() == data
@@ -599,7 +657,7 @@ def test_multi_cache(protocol):
     lurl = fsspec.open_local(
         f"{protocol}::memory://file*",
         mode="rb",
-        **{protocol: {"cache_storage": d2, "same_names": True}}
+        **{protocol: {"cache_storage": d2, "same_names": True}},
     )
     assert all(d2 in u for u in lurl)
     assert all(os.path.basename(f) in ["file0", "file1"] for f in lurl)
@@ -609,7 +667,7 @@ def test_multi_cache(protocol):
     lurl = fsspec.open_files(
         f"{protocol}::memory://file*",
         mode="rb",
-        **{protocol: {"cache_storage": d2, "same_names": True}}
+        **{protocol: {"cache_storage": d2, "same_names": True}},
     )
     with lurl as files:
         for f in files:
@@ -660,19 +718,23 @@ def test_multi_cache_chain(protocol):
 def test_strip(protocol):
     fs = fsspec.filesystem(protocol, target_protocol="memory")
     url1 = "memory://afile"
-    assert fs._strip_protocol(url1) == "afile"
-    assert fs._strip_protocol(protocol + "://afile") == "afile"
-    assert fs._strip_protocol(protocol + "::memory://afile") == "afile"
+    assert fs._strip_protocol(url1) == "/afile"
+    assert fs._strip_protocol(protocol + "://afile") == "/afile"
+    assert fs._strip_protocol(protocol + "::memory://afile") == "/afile"
 
 
 @pytest.mark.parametrize("protocol", ["simplecache", "filecache"])
 def test_cached_write(protocol):
     d = tempfile.mkdtemp()
-    with fsspec.open_files(f"{protocol}::file://{d}/*.out", mode="wb", num=2) as files:
+    ofs = fsspec.open_files(f"{protocol}::file://{d}/*.out", mode="wb", num=2)
+    with ofs as files:
         for f in files:
+            assert isinstance(f, LocalTempFile)
             f.write(b"data")
+            fn = f.name
 
     assert sorted(os.listdir(d)) == ["0.out", "1.out"]
+    assert not os.path.exists(fn)
 
 
 def test_expiry():
@@ -680,7 +742,7 @@ def test_expiry():
 
     d = tempfile.mkdtemp()
     fs = fsspec.filesystem("memory")
-    fn = "afile"
+    fn = "/afile"
     fn0 = "memory://afile"
     data = b"hello"
     with fs.open(fn0, "wb") as f:
@@ -717,3 +779,43 @@ def test_expiry():
     assert fs.open(fn0, mode="rb").read() == data
     detail, _ = fs._check_file(fn0)
     assert detail["time"] - start_time > 0.09
+
+
+def test_equality():
+    """Test sane behaviour for equality and hashing.
+
+    Make sure that different CachingFileSystem only test equal to each other
+    when they should, and do not test equal to the filesystem they rely upon.
+    Similarly, make sure their hashes differ when they should and are equal
+    when they should not.
+
+    Related: GitHub#577, GitHub#578
+    """
+    from fsspec.implementations.local import LocalFileSystem
+
+    lfs = LocalFileSystem()
+    cfs1 = CachingFileSystem(fs=lfs, cache_storage="raspberry")
+    cfs2 = CachingFileSystem(fs=lfs, cache_storage="banana")
+    cfs3 = CachingFileSystem(fs=lfs, cache_storage="banana")
+    assert cfs1 == cfs1
+    assert cfs1 != cfs2
+    assert cfs1 != cfs3
+    assert cfs2 == cfs3
+    assert cfs1 != lfs
+    assert cfs2 != lfs
+    assert cfs3 != lfs
+    assert hash(lfs) != hash(cfs1)
+    assert hash(lfs) != hash(cfs2)
+    assert hash(lfs) != hash(cfs3)
+    assert hash(cfs1) != hash(cfs2)
+    assert hash(cfs1) != hash(cfs2)
+    assert hash(cfs2) == hash(cfs3)
+
+
+def test_str():
+    """Test that the str representation refers to correct class."""
+    from fsspec.implementations.local import LocalFileSystem
+
+    lfs = LocalFileSystem()
+    cfs = CachingFileSystem(fs=lfs)
+    assert "CachingFileSystem" in str(cfs)
