@@ -1,92 +1,14 @@
-import contextlib
 import asyncio
+import io
 import os
-import pytest
-from http.server import BaseHTTPRequestHandler, HTTPServer
 import sys
-import threading
-import fsspec
+import time
 
-requests = pytest.importorskip("requests")
-port = 9898
-data = b"\n".join([b"some test data"] * 1000)
-realfile = "http://localhost:%i/index/realfile" % port
-index = b'<a href="%s">Link</a>' % realfile.encode()
-win = os.name == "nt"
+import pytest
 
-
-class HTTPTestHandler(BaseHTTPRequestHandler):
-    def _respond(self, code=200, headers=None, data=b""):
-        headers = headers or {}
-        headers.update({"User-Agent": "test"})
-        self.send_response(code)
-        for k, v in headers.items():
-            self.send_header(k, str(v))
-        self.end_headers()
-        if data:
-            self.wfile.write(data)
-
-    def do_GET(self):
-        if self.path.rstrip("/") not in [
-            "/index/realfile",
-            "/index/otherfile",
-            "/index",
-        ]:
-            self._respond(404)
-            return
-
-        d = data if self.path in ["/index/realfile", "/index/otherfile"] else index
-        if "Range" in self.headers:
-            ran = self.headers["Range"]
-            b, ran = ran.split("=")
-            start, end = ran.split("-")
-            d = d[int(start) : int(end) + 1]
-        if "give_length" in self.headers:
-            response_headers = {"Content-Length": len(d)}
-            self._respond(200, response_headers, d)
-        elif "give_range" in self.headers:
-            self._respond(200, {"Content-Range": "0-%i/%i" % (len(d) - 1, len(d))}, d)
-        else:
-            self._respond(200, data=d)
-
-    def do_HEAD(self):
-        if "head_ok" not in self.headers:
-            self._respond(405)
-            return
-        d = data if self.path == "/index/realfile" else index
-        if self.path.rstrip("/") not in ["/index/realfile", "/index"]:
-            self._respond(404)
-        elif "give_length" in self.headers:
-            response_headers = {"Content-Length": len(d)}
-            if "zero_length" in self.headers:
-                response_headers["Content-Length"] = 0
-
-            self._respond(200, response_headers)
-        elif "give_range" in self.headers:
-            self._respond(200, {"Content-Range": "0-%i/%i" % (len(d) - 1, len(d))})
-        else:
-            self._respond(200)  # OK response, but no useful info
-
-
-@contextlib.contextmanager
-def serve():
-    server_address = ("", port)
-    httpd = HTTPServer(server_address, HTTPTestHandler)
-    th = threading.Thread(target=httpd.serve_forever)
-    th.daemon = True
-    th.start()
-    try:
-        yield "http://localhost:%i" % port
-    finally:
-        httpd.socket.close()
-        httpd.shutdown()
-        th.join()
-
-
-@pytest.fixture(scope="module")
-def server():
-    with serve() as s:
-        yield s
+import fsspec.asyn
+import fsspec.utils
+from fsspec.tests.conftest import data, reset_files, server, win  # noqa: F401
 
 
 def test_list(server):
@@ -95,10 +17,120 @@ def test_list(server):
     assert out == [server + "/index/realfile"]
 
 
+def test_list_invalid_args(server):
+    with pytest.raises(TypeError):
+        h = fsspec.filesystem("http", use_foobar=True)
+        h.glob(server + "/index/*")
+
+
+def test_list_cache(server):
+    h = fsspec.filesystem("http", use_listings_cache=True)
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+
+def test_list_cache_with_expiry_time_cached(server):
+    h = fsspec.filesystem("http", use_listings_cache=True, listings_expiry_time=30)
+
+    # First, the directory cache is not initialized.
+    assert not h.dircache
+
+    # By querying the filesystem with "use_listings_cache=True",
+    # the cache will automatically get populated.
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+    # Verify cache content.
+    assert len(h.dircache) == 1
+
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+
+def test_list_cache_with_expiry_time_purged(server):
+    h = fsspec.filesystem("http", use_listings_cache=True, listings_expiry_time=0.3)
+
+    # First, the directory cache is not initialized.
+    assert not h.dircache
+
+    # By querying the filesystem with "use_listings_cache=True",
+    # the cache will automatically get populated.
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+    assert len(h.dircache) == 1
+
+    # Verify cache content.
+    assert server + "/index/" in h.dircache
+    assert len(h.dircache.get(server + "/index/")) == 1
+
+    # Wait beyond the TTL / cache expiry time.
+    time.sleep(0.31)
+
+    # Verify that the cache item should have been purged.
+    cached_items = h.dircache.get(server + "/index/")
+    assert cached_items is None
+
+    # Verify that after clearing the item from the cache,
+    # it can get populated again.
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+    cached_items = h.dircache.get(server + "/index/")
+    assert len(cached_items) == 1
+
+
+def test_list_cache_reuse(server):
+    h = fsspec.filesystem("http", use_listings_cache=True, listings_expiry_time=5)
+
+    # First, the directory cache is not initialized.
+    assert not h.dircache
+
+    # By querying the filesystem with "use_listings_cache=True",
+    # the cache will automatically get populated.
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+    # Verify cache content.
+    assert len(h.dircache) == 1
+
+    # Verify another instance without caching enabled does not have cache content.
+    h = fsspec.filesystem("http", use_listings_cache=False)
+    assert not h.dircache
+
+    # Verify that yet another new instance, with caching enabled,
+    # will see the same cache content again.
+    h = fsspec.filesystem("http", use_listings_cache=True, listings_expiry_time=5)
+    assert len(h.dircache) == 1
+
+    # However, yet another instance with a different expiry time will also not have
+    # any valid cache content.
+    h = fsspec.filesystem("http", use_listings_cache=True, listings_expiry_time=666)
+    assert len(h.dircache) == 0
+
+
+def test_ls_raises_filenotfound(server):
+    h = fsspec.filesystem("http")
+
+    with pytest.raises(FileNotFoundError):
+        h.ls(server + "/not-a-key")
+
+
+def test_list_cache_with_max_paths(server):
+    h = fsspec.filesystem("http", use_listings_cache=True, max_paths=5)
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+
+def test_list_cache_with_skip_instance_cache(server):
+    h = fsspec.filesystem("http", use_listings_cache=True, skip_instance_cache=True)
+    out = h.glob(server + "/index/*")
+    assert out == [server + "/index/realfile"]
+
+
 def test_isdir(server):
     h = fsspec.filesystem("http")
     assert h.isdir(server + "/index/")
     assert not h.isdir(server + "/index/realfile")
+    assert not h.isdir(server + "doesnotevenexist")
 
 
 def test_policy_arg(server):
@@ -131,6 +163,11 @@ def test_file_pickle(server):
     # via HTTPFile
     h = fsspec.filesystem("http", headers={"give_length": "true", "head_ok": "true"})
     out = server + "/index/realfile"
+
+    with fsspec.open(out, headers={"give_length": "true", "head_ok": "true"}) as f:
+        pic = pickle.loads(pickle.dumps(f))
+        assert pic.read() == data
+
     with h.open(out, "rb") as f:
         pic = pickle.dumps(f)
         assert f.read() == data
@@ -161,6 +198,8 @@ def test_methods(server):
         {"give_length": "true"},
         {"give_length": "true", "head_ok": "true"},
         {"give_range": "true"},
+        {"give_length": "true", "head_not_auth": "true"},
+        {"give_range": "true", "head_not_auth": "true"},
     ],
 )
 def test_random_access(server, headers):
@@ -177,6 +216,7 @@ def test_random_access(server, headers):
         else:
             with pytest.raises(ValueError):
                 f.seek(5, 1)
+    assert f.closed
 
 
 def test_mapper_url(server):
@@ -219,6 +259,16 @@ def test_multi_download(server, tmpdir):
     assert open(fnb, "rb").read() == data
 
 
+def test_ls(server):
+    h = fsspec.filesystem("http")
+    l = h.ls(server + "/data/20020401/", detail=False)
+    nc = server + "/data/20020401/GRACEDADM_CLSM0125US_7D.A20020401.030.nc4"
+    assert nc in l
+    assert len(l) == 11
+    assert all(u["type"] == "file" for u in h.ls(server + "/data/20020401/"))
+    assert h.glob(server + "/data/20020401/*.nc4") == [nc]
+
+
 def test_mcat(server):
     h = fsspec.filesystem("http", headers={"give_length": "true", "head_ok": "true "})
     urla = server + "/index/realfile"
@@ -227,6 +277,18 @@ def test_mcat(server):
     assert out == {urla: data, urlb: data}
 
 
+def test_cat_file_range(server):
+    h = fsspec.filesystem("http", headers={"give_length": "true", "head_ok": "true "})
+    urla = server + "/index/realfile"
+    assert h.cat(urla, start=1, end=10) == data[1:10]
+    assert h.cat(urla, start=1) == data[1:]
+
+    assert h.cat(urla, start=-10) == data[-10:]
+    assert h.cat(urla, start=-10, end=-2) == data[-10:-2]
+
+    assert h.cat(urla, end=-10) == data[:-10]
+
+
 def test_mcat_cache(server):
     urla = server + "/index/realfile"
     urlb = server + "/index/otherfile"
@@ -240,6 +302,51 @@ def test_mcat_expand(server):
     assert out == {server + "/index/realfile": data}
 
 
+def test_info(server):
+    fs = fsspec.filesystem("http", headers={"give_etag": "true", "head_ok": "true"})
+    info = fs.info(server + "/index/realfile")
+    assert info["ETag"] == "xxx"
+
+
+@pytest.mark.parametrize("method", ["POST", "PUT"])
+def test_put_file(server, tmp_path, method, reset_files):
+    src_file = tmp_path / "file_1"
+    src_file.write_bytes(data)
+
+    dwl_file = tmp_path / "down_1"
+
+    fs = fsspec.filesystem("http", headers={"head_ok": "true", "give_length": "true"})
+    with pytest.raises(FileNotFoundError):
+        fs.info(server + "/hey")
+
+    fs.put_file(src_file, server + "/hey", method=method)
+    assert fs.info(server + "/hey")["size"] == len(data)
+
+    fs.get_file(server + "/hey", dwl_file)
+    assert dwl_file.read_bytes() == data
+
+    src_file.write_bytes(b"xxx")
+    with open(src_file, "rb") as stream:
+        fs.put_file(stream, server + "/hey_2", method=method)
+    assert fs.cat(server + "/hey_2") == b"xxx"
+
+    fs.put_file(io.BytesIO(b"yyy"), server + "/hey_3", method=method)
+    assert fs.cat(server + "/hey_3") == b"yyy"
+
+
+async def get_aiohttp():
+    from aiohttp import ClientSession
+
+    return ClientSession()
+
+
+async def get_proxy():
+    class ProxyClient:
+        pass
+
+    return ProxyClient()
+
+
 @pytest.mark.xfail(
     condition=sys.flags.optimize > 1, reason="no docstrings when optimised"
 )
@@ -258,45 +365,45 @@ def test_async_other_thread(server):
 
     th.daemon = True
     th.start()
-    fs = fsspec.filesystem("http", asynchronous=False, loop=loop)
-    cor = fs._cat([server + "/index/realfile"])
+    fs = fsspec.filesystem("http", asynchronous=True, loop=loop)
+    asyncio.run_coroutine_threadsafe(fs.set_session(), loop=loop).result()
+    url = server + "/index/realfile"
+    cor = fs._cat([url])
     fut = asyncio.run_coroutine_threadsafe(cor, loop=loop)
-    assert fut.result() == [data]
+    assert fut.result() == {url: data}
+    loop.call_soon_threadsafe(loop.stop)
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_async_this_thread(server):
     async def _():
-        loop = asyncio.get_event_loop()
-        fs = fsspec.filesystem("http", asynchronous=True, loop=loop)
+        fs = fsspec.filesystem("http", asynchronous=True)
 
-        # fails because client creation has not yet been awaited
-        assert isinstance(
-            (await fs._cat([server + "/index/realfile"]))[0], RuntimeError
-        )
-        with pytest.raises(RuntimeError):
-            fs.cat([server + "/index/realfile"])
+        session = await fs.set_session()  # creates client
 
-        await fs.set_session()  # creates client
-
-        out = await fs._cat([server + "/index/realfile"])
+        url = server + "/index/realfile"
+        with pytest.raises((NotImplementedError, RuntimeError)):
+            fs.cat([url])
+        out = await fs._cat([url])
         del fs
-        assert out == [data]
+        assert out == {url: data}
+        await session.close()
 
     asyncio.run(_())
 
 
 def _inner_pass(fs, q, fn):
-    # pass the s3 instance, but don't use it; in new process, the instance
+    # pass the FS instance, but don't use it; in new process, the instance
     # cache should be skipped to make a new instance
-    fs = fsspec.filesystem("http")
-    q.put(fs.cat(fn))
+    import traceback
+
+    try:
+        fs = fsspec.filesystem("http")
+        q.put(fs.cat(fn))
+    except Exception:
+        q.put(traceback.format_exc())
 
 
-@pytest.mark.skipif(
-    bool(os.environ.get("CIRUN", "")), reason="CI runs are weird in many ways"
-)
-@pytest.mark.parametrize("method", ["spawn", "forkserver", "fork"])
+@pytest.mark.parametrize("method", ["spawn", "forkserver"])
 def test_processes(server, method):
     import multiprocessing as mp
 
@@ -309,5 +416,25 @@ def test_processes(server, method):
     q = ctx.Queue()
     p = ctx.Process(target=_inner_pass, args=(fs, q, fn))
     p.start()
-    assert q.get() == fs.cat(fn)
+    out = q.get()
+    assert out == fs.cat(fn)
     p.join()
+
+
+@pytest.mark.parametrize("get_client", [get_aiohttp, get_proxy])
+def test_close(get_client):
+    fs = fsspec.filesystem("http", skip_instance_cache=True)
+    fs.close_session(None, asyncio.run(get_client()))
+
+
+@pytest.mark.asyncio
+async def test_async_file(server):
+    fs = fsspec.filesystem("http", asynchronous=True, skip_instance_cache=True)
+    fn = server + "/index/realfile"
+    of = await fs.open_async(fn)
+    async with of as f:
+        out1 = await f.read(10)
+        assert data.startswith(out1)
+        out2 = await f.read()
+        assert data == out1 + out2
+    await fs._session.close()
