@@ -949,7 +949,7 @@ class AbstractFileSystem(metaclass=_Cached):
 
     @classmethod
     def _parent(cls, path):
-        path = cls._strip_protocol(path.rstrip("/"))
+        path = cls._strip_protocol(path)
         if "/" in path:
             parent = path.rsplit("/", 1)[0].lstrip(cls.root_marker)
             return cls.root_marker + parent
