@@ -3,13 +3,12 @@
 import contextlib
 import os
 import pickle
-import sys
 import tempfile
 
 import pytest
 
 import fsspec
-from fsspec.implementations.memory import MemoryFileSystem, MemoryFile
+from fsspec.implementations.memory import MemoryFile, MemoryFileSystem
 
 
 def test_idempotent():
@@ -33,15 +32,14 @@ def test_pickle():
 
 
 def test_class_methods():
-    assert MemoryFileSystem._strip_protocol("memory:stuff") == "stuff"
-    assert MemoryFileSystem._strip_protocol("memory://stuff") == "stuff"
-    assert MemoryFileSystem._strip_protocol("stuff") == "stuff"
+    assert MemoryFileSystem._strip_protocol("memory://stuff") == "/stuff"
+    assert MemoryFileSystem._strip_protocol("stuff") == "/stuff"
     assert MemoryFileSystem._strip_protocol("other://stuff") == "other://stuff"
 
     assert MemoryFileSystem._get_kwargs_from_urls("memory://user@thing") == {}
 
 
-def test_get_put(tmpdir):
+def test_get_put(tmpdir, m):
     tmpdir = str(tmpdir)
     fn = os.path.join(tmpdir, "one")
     open(fn, "wb").write(b"one")
@@ -95,19 +93,21 @@ def test_get_put(tmpdir):
     assert open(fn, "rb").read() == b"one"
 
 
-def test_du():
+def test_du(m):
     fs = MemoryFileSystem()
-    fs.store = {
-        "/dir/afile": MemoryFile(fs, "/afile", b"a"),
-        "/dir/dirb/afile": MemoryFile(fs, "/afile", b"bb"),
-        "/dir/dirb/bfile": MemoryFile(fs, "/afile", b"ccc"),
-    }
+    fs.store.update(
+        {
+            "/dir/afile": MemoryFile(fs, "/afile", b"a"),
+            "/dir/dirb/afile": MemoryFile(fs, "/afile", b"bb"),
+            "/dir/dirb/bfile": MemoryFile(fs, "/afile", b"ccc"),
+        }
+    )
     assert fs.du("/dir") == 6
     assert fs.du("/dir", total=False)["/dir/dirb/afile"] == 2
     assert fs.du("/dir", maxdepth=0) == 1
 
 
-def test_head_tail():
+def test_head_tail(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"I had a nice big cabbage")
@@ -115,7 +115,7 @@ def test_head_tail():
     assert fs.tail("/myfile", 7) == b"cabbage"
 
 
-def test_move():
+def test_move(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"I had a nice big cabbage")
@@ -125,7 +125,7 @@ def test_move():
     assert isinstance(fs.ukey("/otherfile"), str)
 
 
-def test_recursive_get_put(tmpdir):
+def test_recursive_get_put(tmpdir, m):
     fs = MemoryFileSystem()
     os.makedirs(f"{tmpdir}/nest")
     for file in ["one", "two", "nest/other"]:
@@ -141,17 +141,17 @@ def test_recursive_get_put(tmpdir):
             f.read() == b"data"
 
 
-def test_pipe_cat():
+def test_pipe_cat(m):
     fs = MemoryFileSystem()
     fs.pipe("afile", b"contents")
     assert fs.cat("afile") == b"contents"
 
-    data = {"bfile": b"more", "cfile": b"stuff"}
+    data = {"/bfile": b"more", "/cfile": b"stuff"}
     fs.pipe(data)
     assert fs.cat(list(data)) == data
 
 
-def test_read_block_delimiter():
+def test_read_block_delimiter(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"some\n" b"lines\n" b"of\n" b"text")
@@ -163,7 +163,7 @@ def test_read_block_delimiter():
     assert fs.read_block("/myfile", 0, None) == fs.cat("/myfile")
 
 
-def test_open_text():
+def test_open_text(m):
     fs = MemoryFileSystem()
     with fs.open("/myfile", "wb") as f:
         f.write(b"some\n" b"lines\n" b"of\n" b"text")
@@ -190,7 +190,7 @@ def test_chained_fs():
 
 @pytest.mark.xfail(reason="see issue #334", strict=True)
 def test_multilevel_chained_fs():
-    """This test reproduces intake/filesystem_spec#334"""
+    """This test reproduces fsspec/filesystem_spec#334"""
     import zipfile
 
     d1 = tempfile.mkdtemp()
@@ -214,9 +214,8 @@ def test_multilevel_chained_fs():
             assert f.read().decode("utf-8") == f.name
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason="no seek in old zipfile")
 def test_multilevel_chained_fs_zip_zip_file():
-    """This test reproduces intake/filesystem_spec#334"""
+    """This test reproduces fsspec/filesystem_spec#334"""
     import zipfile
 
     d1 = tempfile.mkdtemp()
@@ -324,3 +323,11 @@ def test_chained_fo():
     with of[0] as f:
         assert f.read() == b"test"
     assert "afile" in os.listdir(d3)
+
+
+def test_url_to_fs():
+    url = "memory://a.txt"
+    fs, url2 = fsspec.core.url_to_fs(url)
+
+    assert isinstance(fs, MemoryFileSystem)
+    assert url2 == "/a.txt"
